#!/usr/bin/lua
require "lunit"
require "lfs"
log = require "syslog"
json = require "cjson"
local socket = require "socket"
socket.unix = require "socket.unix"
local currentdir = lfs.currentdir()
package.path = package.path..";"..currentdir.."/etc/?.lua;"..currentdir.."/share/?.lua;"..currentdir.."/etc/rules/?.lua"
require "synergy_inc"
require "object_inc"
mklive:init_socket()

-- Tests functionality of nagios rules
module("nagios-object-test", package.seeall, lunit.testcase)

function setup()
	require "config"
end

-- Tests functionality of host rule
function test_nagios_host()
	local status, nagios_host_rule, msg = pcall(require, "rules/repo/host")
	assert_table(nagios_host_rule.host.params, "Params are not valid.")
	assert_function(nagios_host_rule.host.rule, "Host rule should be a valid function")
	local host_status = nagios_host_rule.host.rule({}, "monitor", "all")
	assert_table(host_status, "Rule returned unexpected results")
	assert_number(host_status[1].status, "Host rule did not return a valid state")
	local s, nil_status = pcall(nagios_host_rule.host.rule, {}, "nil", "all")
	assert_table(nil_status, "Rule crashes if host can't be found")
end

-- Tests functionality of service rule
function test_nagios_service()
	local status, nagios_service_rule, msg = pcall(require, "rules/repo/service")
	assert_table(nagios_service_rule.service.params, "Params are not valid")
	assert_function(nagios_service_rule.service.rule, "Service rule should be a valid function")
	local service_status = nagios_service_rule.service.rule({}, "monitor&ssh", "all")
	assert_table(service_status, "Rule returned unexpected results")
	assert_number(service_status[1].status, "Service rule did not return a valid state")
	local s, nil_status = pcall(nagios_service_rule.service.rule, {}, "nil&nil", "all")
	assert_table(nil_status, "Rule crashes if host can't be found")
end

-- Tests functionality of hostgroup rule
function test_nagios_hostgroup()
	local status, nagios_hostgroup_rule, msg = pcall(require, "rules/repo/host_group")
	assert_table(nagios_hostgroup_rule.host_group.params, "Params are not valid")
	assert_function(nagios_hostgroup_rule.host_group.rule, "Hostgroup rule should be a valid function")
	local s, hostgroup_status = pcall(nagios_hostgroup_rule.host_group.rule, {}, "unix-servers", "all")
	assert_table(hostgroup_status, "Rule returned unexpected results")
	assert_number(hostgroup_status[1].status, "Hostgroup rule did not return a valid state")
	local s, nil_status = pcall(nagios_hostgroup_rule.host_group.rule, {}, "nil", "all")
	assert_table(nil_status, "Rule crashes if hostgroup can't be found")
end

-- Tests functionality of hosts of group rule
function test_nagios_hosts_of_group()
	local status, nagios_hostsofgroup_rule, msg = pcall(require, "rules/repo/hosts_of_group")
	assert_table(nagios_hostsofgroup_rule.hosts_of_group.params, "Params are not valid")
	assert_function(nagios_hostsofgroup_rule.hosts_of_group.rule, "Hosts of group rule should be a valid function")
	local s, hostgroup_status = pcall(nagios_hostsofgroup_rule.hosts_of_group.rule, {}, "unix-servers", "all")
	assert_table(hostgroup_status, "Rule returned unexpected results")
	assert_number(hostgroup_status[1].status, "Hostgroup rule did not return a valid state")
	local s, nil_status = pcall(nagios_hostsofgroup_rule.hosts_of_group.rule, {}, "nil", "all")
	assert_table(nil_status, "Rule crashes if hostgroup can't be found")
end

-- Tests functionality of servicegroup rule
function test_nagios_servicegroup()
	local status, nagios_servicegroup_rule, msg = pcall(require, "rules/repo/service_group")
	assert_table(nagios_servicegroup_rule.service_group.params, "Params are not valid")
	assert_function(nagios_servicegroup_rule.service_group.rule, "Servicegroup rule should be a valid function")
	-- @TODO: Create a servicegroup in testconfig.
	local s, servicegroup_status = pcall(nagios_servicegroup_rule.service_group.rule, {}, "bogus-servicegroup", "all")
	assert_table(servicegroup_status, "Rule returned unexpected results")
	assert_number(servicegroup_status[1].status, "Host rule did not return a valid state")
	local s, nil_status = pcall(nagios_servicegroup_rule.service_group.rule, {}, "nil", "all")
	assert_table(nil_status, "Rule crashes if servicegroup can't be found")
end

-- Tests functionality of services of group rule
function test_nagios_services_of_group()
	local status, nagios_servicesofgroup_rule, msg = pcall(require, "rules/repo/services_of_group")
	assert_table(nagios_servicesofgroup_rule.services_of_group.params, "Params are not valid")
	assert_function(nagios_servicesofgroup_rule.services_of_group.rule, "Services of group rule should be a valid function")
	-- @TODO: Create a servicegroup in testconfig.
	local s, servicegroup_status = pcall(nagios_servicesofgroup_rule.services_of_group.rule, {}, "bogus-servicegroup", "all")
	assert_table(servicegroup_status, "Rule returned unexpected results")
	assert_number(servicegroup_status[1].status, "Host rule did not return a valid state")
	local s, nil_status = pcall(nagios_servicesofgroup_rule.services_of_group.rule, {}, "nil", "all")
	assert_table(nil_status, "Rule crashes if servicegroup can't be found")
end

-- Test functionality of host services rule
function test_nagios_hostservices()
	local status, nagios_host_services_rule, msg = pcall(require, "rules/repo/host_services")
	assert_table(nagios_host_services_rule.host_services.params, "Params are not valid.")
	assert_function(nagios_host_services_rule.host_services.rule, "Host_services rule should be a valid function")
	-- @TODO: Create a servicegroup in testconfig.
	local host_services_status = nagios_host_services_rule.host_services.rule({}, "monitor", "all")
	assert_table(host_services_status, "Rule returned unexpected results")
	assert_number(host_services_status[1].status, "Host services rule did not return a valid state")
	local s, nil_status = pcall(nagios_host_services_rule.host_services.rule, {}, "nil", "all")
	assert_table(nil_status, "Rule crashes if host can't be found")
end

-- Tests functionality of synergy_group rules
module("group-rule-test", package.seeall, lunit.testcase)

local test_rules = {
	at_least_num_ok = { name = "foo", intent = rule.at_least(2,1,"num"), components = { { name = "c1", intent = rule.constant("OK") }, { name = "c2", intent = rule.constant("OK") }, { name = "c3", intent = rule.constant("WARNING") } } },
	at_least_num_warning = { name = "foo", intent = rule.at_least(2,1,"num"), components = { { name = "c1", intent = rule.constant("OK") }, { name = "c2", intent = rule.constant("WARNING") }, { name = "c3", intent = rule.constant("WARNING") } } },
	at_least_num_critical = { name = "foo", intent = rule.at_least(2,1,"num"), components = { { name = "c1", intent = rule.constant("WARNING") }, { name = "c2", intent = rule.constant("WARNING") }, { name = "c3", intent = rule.constant("WARNING") } } },
	at_least_percent_ok = { name = "foo", intent = rule.at_least(60,30,"%"), components = { { name = "c1", intent = rule.constant("OK") }, { name = "c2", intent = rule.constant("OK") }, { name = "c3", intent = rule.constant("WARNING") } } },
	at_least_percent_warning = { name = "foo", intent = rule.at_least(60,30,"%"), components = { { name = "c1", intent = rule.constant("OK") }, { name = "c2", intent = rule.constant("WARNING") }, { name = "c3", intent = rule.constant("WARNING") } } },
	at_least_percent_critical = { name = "foo", intent = rule.at_least(60,30,"%"), components = { { name = "c1", intent = rule.constant("CRITICAL") }, { name = "c2", intent = rule.constant("WARNING") }, { name = "c3", intent = rule.constant("WARNING") } } },
	at_most_num_ok = { name = "foo", intent = rule.at_most(2,3,"num"), components = { { name = "c1", intent = rule.constant("OK") }, { name = "c2", intent = rule.constant("OK") }, { name = "c3", intent = rule.constant("WARNING") } } },
	at_most_num_warning = { name = "foo", intent = rule.at_most(2,3,"num"), components = { { name = "c1", intent = rule.constant("OK") }, { name = "c2", intent = rule.constant("WARNING") }, { name = "c3", intent = rule.constant("WARNING") } } },
	at_most_num_critical = { name = "foo", intent = rule.at_most(2,3,"num"), components = { { name = "c1", intent = rule.constant("WARNING") }, { name = "c2", intent = rule.constant("WARNING") }, { name = "c3", intent = rule.constant("WARNING") } } },
	at_most_percent_ok = { name = "foo", intent = rule.at_most(40,70,"%"), components = { { name = "c1", intent = rule.constant("OK") }, { name = "c2", intent = rule.constant("OK") }, { name = "c3", intent = rule.constant("WARNING") } } },
	at_most_percent_warning = { name = "foo", intent = rule.at_most(40,70,"%"), components = { { name = "c1", intent = rule.constant("OK") }, { name = "c2", intent = rule.constant("WARNING") }, { name = "c3", intent = rule.constant("WARNING") } } },
	at_most_percent_critical = { name = "foo", intent = rule.at_most(40,70,"%"), components = { { name = "c1", intent = rule.constant("CRITICAL") }, { name = "c2", intent = rule.constant("WARNING") }, { name = "c3", intent = rule.constant("WARNING") } } },
	simple_at_least_num_ok = { name = "foo", intent = rule.simple_at_least(2,"num"), components = { { name = "c1", intent = rule.constant("OK") }, { name = "c2", intent = rule.constant("OK") }, { name = "c3", intent = rule.constant("WARNING") } } },
	simple_at_least_num_warning = { name = "foo", intent = rule.simple_at_least(2,"num"), components = { { name = "c1", intent = rule.constant("OK") }, { name = "c2", intent = rule.constant("WARNING") }, { name = "c3", intent = rule.constant("WARNING") } } },
	simple_at_least_percent_ok = { name = "foo", intent = rule.simple_at_least(60,"%"), components = { { name = "c1", intent = rule.constant("OK") }, { name = "c2", intent = rule.constant("OK") }, { name = "c3", intent = rule.constant("WARNING") } } },
	simple_at_least_percent_warning = { name = "foo", intent = rule.simple_at_least(60,"%"), components = { { name = "c1", intent = rule.constant("OK") }, { name = "c2", intent = rule.constant("WARNING") }, { name = "c3", intent = rule.constant("WARNING") } } },
	best_state = { name = "Best", intent = rule.best_state(), components = { { name = "c1", intent = rule.constant("WARNING") } } },
	worst_state = { name = "Worst", intent = rule.worst_state(), components = { { name = "c1", intent = rule.constant("WARNING") } } },
	scores_ok = { name = "Scores", intent = rule.scores(2,3), components = { { name = "c1", intent = rule.constant("WARNING") } } },
	scores_warning = { name = "Scores", intent = rule.scores(2,3), components = { { name = "c1", intent = rule.constant("WARNING") }, { name = "c2", intent = rule.constant("WARNING") } } },
	scores_critical = { name = "Scores", intent = rule.scores(2,3), components = { { name = "c1", intent = rule.constant("WARNING") }, { name = "c2", intent = rule.constant("CRITICAL") } } }
}


-- Tests at least x items must be ok
function test_at_least()
	local status, at_least, msg = pcall(require, "rules/repo/at_least")
	assert_table(at_least.at_least.params, "Params are not valid")
	assert_function(at_least.at_least.rule, "At_least rule should be a valid function")

	-- Test for a returned OK state using num
	local s, result = pcall(execute, test_rules.at_least_num_ok, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(0, result[1].status, "Expected status to return 0 got "..result[1].status)

	-- Test for a returned WARNING state using num
	local s, result = pcall(execute, test_rules.at_least_num_warning, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(1, result[1].status, "Expected status to return 1 got "..result[1].status)

	-- Test for a returned CRITICAL using num
	local s, result = pcall(execute, test_rules.at_least_num_critical, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(2, result[1].status, "Expected status to return 2 got "..result[1].status)

	-- Test for a returned OK using percent
	local s, result = pcall(execute, test_rules.at_least_percent_ok, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(0, result[1].status, "Expected status to return 0 got "..result[1].status)

	-- Test for a returned WARNING state using percent
	local s, result = pcall(execute, test_rules.at_least_percent_warning, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(1, result[1].status, "Expected status to return 1 got "..result[1].status)

	-- Test for a returned CRITICAL state using percent
	local s, result = pcall(execute, test_rules.at_least_percent_critical, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(2, result[1].status, "Expected status to return 2 got "..result[1].status)
end

-- Tests at most x items can have problems
function test_at_most()
	local status, at_most, msg = pcall(require, "rules/repo/at_most")
	assert_table(at_most.at_most.params, "Params are not valid")
	assert_function(at_most.at_most.rule, "At_most rule should be a valid function")

	-- Test for a returned OK state using num
	local s, result = pcall(execute, test_rules.at_most_num_ok, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(0, result[1].status, "Expected status to return 0 got "..result[1].status)

	-- Test for a returned WARNING state using num
	local s, result = pcall(execute, test_rules.at_most_num_warning, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(1, result[1].status, "Expected status to return 1 got "..result[1].status)

	-- Test for a returned CRITICAL using num
	local s, result = pcall(execute, test_rules.at_most_num_critical, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(2, result[1].status, "Expected status to return 2 got "..result[1].status)

	-- Test for a returned OK using percent
	local s, result = pcall(execute, test_rules.at_most_percent_ok, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(0, result[1].status, "Expected status to return 0 got "..result[1].status)

	-- Test for a returned WARNING state using percent
	local s, result = pcall(execute, test_rules.at_most_percent_warning, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(1, result[1].status, "Expected status to return 1 got "..result[1].status)

	-- Test for a returned CRITICAL state using percent
	local s, result = pcall(execute, test_rules.at_most_percent_critical, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(2, result[1].status, "Expected status to return 2 got "..result[1].status)
end

-- Tests simple at least x items must be ok
function test_simple_at_least()
	local status, simple_at_least, msg = pcall(require, "rules/repo/simple_at_least")
	assert_table(simple_at_least.simple_at_least.params, "Params are not valid")
	assert_function(simple_at_least.simple_at_least.rule, "Simple_at_least rule should be a valid function")

	-- Test for a returned OK state using num
	local s, result = pcall(execute, test_rules.simple_at_least_num_ok, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(0, result[1].status, "Expected status to return 0 got "..result[1].status)

	-- Test for a returned WARNING state using num
	local s, result = pcall(execute, test_rules.simple_at_least_num_warning, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(1, result[1].status, "Expected status to return 1 got "..result[1].status)

	-- Test for a returned OK using percent
	local s, result = pcall(execute, test_rules.simple_at_least_percent_ok, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(0, result[1].status, "Expected status to return 0 got "..result[1].status)

	-- Test for a returned WARNING state using percent
	local s, result = pcall(execute, test_rules.simple_at_least_percent_warning, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(1, result[1].status, "Expected status to return 1 got "..result[1].status)
end

-- Tests best state among objects
function test_best_state()
	local status, best_state, msg = pcall(require, "rules/repo/best_state")
	assert_table(best_state.best_state.params, "Params are not valid")
	assert_function(best_state.best_state.rule, "Best_state should be a valid function")
	local s, result = pcall(execute, test_rules.best_state, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(1, result[1].status, "Expected status to return 1 got "..result[1].status)
	assert_equal("The best state is WARNING", result[1].msg)
end

-- Tests worst state among objects
function test_worst_state()
	local status, worst_state, msg = pcall(require, "rules/repo/worst_state")
	assert_table(worst_state.worst_state.params, "Params are not valid")
	assert_function(worst_state.worst_state.rule, "Worst_state rule should be a valid function")
	local s, result = pcall(execute, test_rules.worst_state, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(1, result[1].status, "Expected status to return 1 got "..result[1].status)
	assert_equal("The worst state is WARNING", result[1].msg)
end

function test_scores()
	local status, scores, msg = pcall(require, "rules/repo/scores")
	assert_table(scores.scores.params, "Params are not valid")
	assert_function(scores.scores.rule, "Scores rule should be a valid function")

	-- Test for a returned OK state
	local s, result = pcall(execute, test_rules.scores_ok, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(0, result[1].status, "Expected status to return 0 got "..result[1].status)

	-- Test for a returned WARNING STATE
	local s, result = pcall(execute, test_rules.scores_warning, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(1, result[1].status, "Expected status to return 1 got "..result[1].status)

	-- Test for a returned CRITICAL state
	local s, result = pcall(execute, test_rules.scores_critical, "", false)
	assert_table(result, "Execution of rule returned unexpected results")
	assert_equal(2, result[1].status, "Expected status to return 2 got "..result[1].status)
end

-- Tests functionality of demo objects
module("test-demo-objects", package.seeall, lunit.testcase)

-- Tests random object
function test_random()
	local status, random, msg = pcall(require, "rules/repo/random")
	assert_table(random.random.params, "Params are not valid")
	assert_function(random.random.rule, "Random rule is not a valid function")
	local s, random_status = pcall(random.random.rule, {})
	assert_table(random_status, "Rule returned unexpected results")
	assert_number(random_status[1].status, "Random did not return a valid status result got: "..random_status[1].status)
	if random_status[1].status > 2 then
		fail("Random returned a status higher than 2")
	end
end

-- Tests contant object
function test_constant()
	local states = {"OK", "WARNING", "CRITICAL"}
	local status, constant, msg = pcall(require, "rules/repo/constant")
	assert_table(constant.constant.params, "Params are not valid")
	assert_function(constant.constant.rule, "Constant rule is not a valid function")
	-- OK state
	local s, constant_ok = pcall(constant.constant.rule, {}, "OK")
	assert_equal(0, constant_ok[1].status, "Expected an OK state but got: "..tostring(states[constant_ok[1].status]))
	-- WARNING state
	local s, constant_wr = pcall(constant.constant.rule, {}, "WARNING")
	assert_equal(1, constant_wr[1].status, "Expected a WARNING state but got: "..tostring(states[constant_wr[1].status]))
	-- CRITICAL state
	local s, constant_cr = pcall(constant.constant.rule, {}, "CRITICAL")
	assert_equal(2, constant_cr[1].status, "Expected a CRITICAL state but got: "..tostring(states[constant_cr[1].status]))
end

function teardown()
end
